local _, addon = ...

local spells = addon.spells[select(2, UnitClass("player"))]

-- don't load if there are no reagents associated to our character class
if not spells then return end

TITAN_REAGENTTRACKER_ID = "ReagentTracker"

local L = LibStub("AceLocale-3.0"):GetLocale("Titan", true)

-- store spells that are known here later
local possessed = {}


local function newReagent(parent, i)
	local btn = CreateFrame("Button", "TitanPanelReagentTrackerReagent"..i, parent, "TitanPanelChildButtonTemplate")
	btn:SetSize(16, 16)
	btn:SetPoint("LEFT")
	btn:SetPushedTextOffset(0, 0)
	
	local icon = btn:CreateTexture()
	icon:SetSize(16, 16)
	icon:SetPoint("LEFT")
	btn:SetNormalTexture(icon)
	btn.icon = icon
	
	local text = btn:CreateFontString(nil, nil, "GameFontHighlightSmall")
	text:SetPoint("LEFT", icon, "RIGHT", 2, 1)
	text:SetJustifyH("LEFT")
	btn:SetFontString(text)
	
	return btn
end


local function onUpdate(self, elapsed)
	if self.refreshReagents then
		self:RefreshReagents()
		self.refreshReagents = false
	end
	self:UpdateButton()
	TitanPanelButton_UpdateTooltip(self)
	self:SetScript("OnUpdate", nil)
end


addon = CreateFrame("Button", "TitanPanelReagentTrackerButton", CreateFrame("Frame", nil, UIParent), "TitanPanelButtonTemplate")
addon:SetSize(16, 16)
addon:SetPushedTextOffset(0, 0)
addon:RegisterEvent("PLAYER_LOGIN")
addon:RegisterEvent("LEARNED_SPELL_IN_TAB")
addon:SetScript("OnEvent", function(self, event, ...)
	if event == "PLAYER_LOGIN" then
		self:RefreshReagents()
		self:UpdateButton()
		TitanPanelButton_UpdateTooltip(self)
		self:RegisterEvent("BAG_UPDATE")
	else
		-- update on next frame to prevent redundant CPU processing from event spamming
		self.refreshReagents = event == "LEARNED_SPELL_IN_TAB"
		self:SetScript("OnUpdate", onUpdate)
		return
	end
end)

local text = addon:CreateFontString(nil, nil, "GameFontNormalSmall")
text:SetPoint("LEFT", 0, 1)
text:SetText("No reagents")
addon:SetFontString(text)
addon.label = text


addon.registry = {
	id = TITAN_REAGENTTRACKER_ID,
	version = GetAddOnMetadata("TitanReagentTracker", "Version"),
	menuText = "Reagent Tracker",
	tooltipTitle = "Reagent Tracker Info", 
	tooltipTextFunction = "TitanPanelReagentTracker_GetTooltipText",
	savedVariables = {
		ShowSpellIcons = false,
	}
}


local buttons = {}

for i = 1, #spells do
	buttons[i] = newReagent(addon, i)
	addon.registry.savedVariables["TrackReagent"..i] = (i == 1)
	possessed[i] = {}
end


local queryTooltip = CreateFrame("GameTooltip", "TitanReagentTrackerTooltip", nil, "GameTooltipTemplate")
queryTooltip:SetOwner(UIParent, "ANCHOR_NONE")
queryTooltip:SetScript("OnTooltipSetItem", function(self)
	if TitanReagentTrackerTooltipTextLeft1:GetText() ~= RETRIEVING_ITEM_INFO then
		addon:RefreshReagents()
		addon:UpdateButton()
		TitanPanelButton_UpdateTooltip(addon)
	end
end)


function addon:RefreshReagents()
	for index, buff in ipairs(spells) do
		local possessed = possessed[index]
		wipe(possessed)
		for index, spell in ipairs(buff.spells) do
			if IsSpellKnown(spell) then
				local reagentID = buff.reagent
				local reagentName = GetItemInfo(reagentID)
				if not reagentName then
					queryTooltip:SetHyperlink("item:"..reagentID)
					return
				end
				-- add this spell to our "known spells" table
				-- if this reagent is already tracked through another spell, we'll want to hide it
				possessed.disabled = self:IsReagentTracked(reagentName)
				possessed.reagentName = reagentName
				possessed.reagentIcon = GetItemIcon(reagentID)
				possessed.spellIcon = GetSpellTexture(spell)
				break
			end
		end
	end
end


function addon:IsReagentTracked(reagent)
	for i, v in ipairs(possessed) do
		-- ignore already ignored reagents
		if v.reagentName == reagent and not v.disabled then
			return true
		end
	end
end


function addon:UpdateButton()
	local tracking
	local totalWidth = 0
	local offset = 0
	
	for i, buff in ipairs(possessed) do
		local button = buttons[i]
		local nextButton = buttons[i + 1]
		local nextAnchor = "LEFT"
		local nextOffset = 0
		
		-- show/hide reagent trackers
		if buff.reagentName and not buff.disabled and TitanGetVar(TITAN_REAGENTTRACKER_ID, "TrackReagent"..i) then
			local icon = button.icon
			button:Show()
			-- display spell or reagent icon
			if TitanGetVar(TITAN_REAGENTTRACKER_ID, "ShowSpellIcons") then
				icon:SetTexture(buff.spellIcon)
			else
				icon:SetTexture(buff.reagentIcon)
			end
			
			-- current number of reagents
			button:SetText(GetItemCount(buff.reagentName))
			
			if nextButton then
				nextAnchor = "RIGHT"
				nextOffset = 6
			end
			
			button:SetWidth(icon:GetWidth() + button:GetTextWidth())
			totalWidth = totalWidth + button:GetWidth()
			
			offset = offset + 1
			tracking = true
		else
			button:Hide()
		end
		
		-- fix offset to next reagent tracker
		if nextButton then
			nextButton:SetPoint("LEFT", button, nextAnchor, nextOffset, 0)
		end
	end
	
	-- show label if no tracking is enabled
	local none = self.label
	if tracking then
		none:Hide()
	else
		none:Show()
		totalWidth = none:GetWidth() + 8
	end
	
	-- adjust width so other plugins are properly offset
	self:SetWidth(totalWidth + ((offset - 1) * 8))
end


function addon:ToggleVar()
	TitanToggleVar(TITAN_REAGENTTRACKER_ID, self.value)
	addon:UpdateButton()
end


function TitanPanelRightClickMenu_PrepareReagentTrackerMenu()
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[TITAN_REAGENTTRACKER_ID].menuText)
	
	local info = {}
	
	-- add meny entry for each possessed spell
	for index, buff in ipairs(possessed) do
		local reagent = buff.reagentName
		if reagent then
			info.text = "Track "..reagent
			info.value = "TrackReagent"..index
			info.func = addon.ToggleVar
			info.checked = TitanGetVar(TITAN_REAGENTTRACKER_ID, "TrackReagent"..index)
			info.isNotRadio = true
			info.disabled = buff.disabled
			info.keepShownOnClick = 1
			UIDropDownMenu_AddButton(info)
		end
	end
	
	TitanPanelRightClickMenu_AddSpacer()
	
	info.text = "Show spell icons"
	info.value = "ShowSpellIcons"
	info.func = addon.ToggleVar
	info.checked = TitanGetVar(TITAN_REAGENTTRACKER_ID, "ShowSpellIcons")
	info.isNotRadio = true
	info.keepShownOnClick = 1
	UIDropDownMenu_AddButton(info)
	
	TitanPanelRightClickMenu_AddSpacer()
	
	TitanPanelRightClickMenu_AddCommand(L["TITAN_PANEL_MENU_HIDE"], TITAN_REAGENTTRACKER_ID, TITAN_PANEL_MENU_FUNC_HIDE)
end


function TitanPanelReagentTracker_GetTooltipText()
	local tooltipText = " "
	
	-- reagent info in tooltip
	for index, buff in ipairs(possessed) do
		local reagent = buff.reagentName
		if reagent and not buff.disabled and TitanGetVar(TITAN_REAGENTTRACKER_ID, "TrackReagent"..index) then
			tooltipText = format("%s\n%s\t%s", tooltipText, reagent, TitanUtils_GetHighlightText(GetItemCount(reagent)))
		end
	end
	
	if #tooltipText > 1 then
		return tooltipText
	else
		return " \nNo reagents tracked."
	end
end